package com.jpsoft.bus.modules.bus.callback;

import cn.hutool.core.lang.UUID;
import com.alipay.api.domain.DeviceInfo;
import com.jpsoft.bus.modules.base.entity.MergeOrderInfo;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.service.MergeOrderInfoService;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.bus.entity.*;
import com.jpsoft.bus.modules.bus.service.*;
import com.jpsoft.bus.modules.common.utils.CommonUtil;
import com.jpsoft.gps.callback.GpsDataCallback;
import com.jpsoft.gps.utils.GPSUtil;
import com.jpsoft.bus.config.GpsParamConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.Minutes;
import org.joda.time.Seconds;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Slf4j
@Component
public class GpsDataCallbackImpl implements GpsDataCallback {
    @Autowired
    private ValueOperations<String, Object> valueOperations;

    @Autowired
    private GpsDataInfoService gpsDataInfoService;

    @Autowired
    private GpsService gpsService;

    @Autowired
    private UserInfoService userInfoService;

    @Autowired
    private PassengerMessageService passengerMessageService;

    @Autowired
    private MergeOrderInfoService mergeOrderInfoService;

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private StationInfoService stationInfoService;

    @Autowired
    private PassengerInfoService passengerInfoService;

    @Autowired
    private ShiftInfoService shiftInfoService;

    @Autowired
    private VehicleInfoService vehicleInfoService;

    @Autowired
    private GpsParamConfig gpsParamConfig;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Override
    public void receive(String deviceNo, String longitude, String latitude,String rawData,boolean manual) {
        log.warn("保存实时数据:{},{},{}", deviceNo, longitude, latitude);

        valueOperations.set(deviceNo + "_gps", longitude + "," + latitude, 1, TimeUnit.HOURS);

        if (Double.valueOf(longitude) > 0 && Double.valueOf(latitude) > 0) {
            String histKey = deviceNo+"_history_time";

            //更新车辆经纬度
            VehicleInfo v = vehicleInfoService.findByGpsDeviceNo(deviceNo);

            Integer intervalSeconds = v.getTrailInterval();

            if (intervalSeconds==null){
                intervalSeconds = 60; //默认为60秒
            }

            Long time = (Long)valueOperations.get(histKey);

            boolean allowInsert = false;

            if (time==null){
                allowInsert = true;
            }
            else{
                DateTime createTime = new DateTime(time);

                int seconds = Seconds.secondsBetween(createTime, DateTime.now()).getSeconds();

                if (seconds>=intervalSeconds){
                    allowInsert = true;
                }
            }

            GpsDataInfo gpsDataInfo0 = null;
            if(allowInsert) {
                log.warn("保存历史数据:{}", deviceNo);
                GpsDataInfo gpsDataInfo = new GpsDataInfo();
                gpsDataInfo.setId(UUID.randomUUID().toString());
                gpsDataInfo.setDeviceNo(deviceNo);
                gpsDataInfo.setCreateTime(new Date());
                gpsDataInfo.setLatitude(latitude);
                gpsDataInfo.setLongitude(longitude);
                gpsDataInfo.setDelFlag(false);


                gpsDataInfoService.insert(gpsDataInfo);
                gpsDataInfo0 = gpsDataInfo;

                valueOperations.set(histKey, (new Date()).getTime(), 1, TimeUnit.HOURS);
            }

            if (v != null) {
                log.warn("更新车辆经纬度:{}->({},{})", v.getLicensePlateNumber(), longitude, latitude);

                String lbsKey = "lbs_" + deviceNo;

                String address = (String) valueOperations.get(lbsKey);

                //10分钟进行一次逆向地址解析
                if (StringUtils.isEmpty(address)) {
                    address = GPSUtil.regeo(longitude, latitude, gpsParamConfig.getKey());
                    valueOperations.set(lbsKey, address, 10, TimeUnit.MINUTES);
                }

                //上一次的经纬度
                String lastLongitude = v.getLongitude();
                String lastLatitude = v.getLatitude();

                v.setLatestAddress(address);
                v.setLongitude(longitude);//经度
                v.setLatitude(latitude);//纬度
                v.setRawData(rawData);

                vehicleInfoService.updateGps(v);

                //更新车辆的当前站点
                try{
                    List<ShiftInfo> shiftInfoList = shiftInfoService.findByVehicleIdAndStatus(v.getId(),"1");

                    if (shiftInfoList.size()>0){
                        ShiftInfo shiftInfo = shiftInfoList.get(0);
                        String stationId = gpsService.getLatelyStation(v.getId(),manual);

                        shiftInfoService.stationChange(shiftInfo, stationId);

                        if (gpsDataInfo0 != null){
                            gpsDataInfo0.setStationId(shiftInfo.getCurrentStationId());
                            gpsDataInfoService.update(gpsDataInfo0);
                        }

                        //判断车辆和当前站点的位置关系
                        int radius = 100;
                        StationInfo currentStation = stationInfoService.get(stationId);

                        if (currentStation.getRadius() != null){
                            radius = currentStation.getRadius();
                        }

                        double distance1 = gpsService.calculateDistance(lastLongitude,lastLatitude,currentStation.getLongitude(),currentStation.getLatitude());
                        double distance2 = gpsService.calculateDistance(v.getLongitude(),v.getLatitude(),currentStation.getLongitude(),currentStation.getLatitude());

                        if (distance2<=radius){
                            //已到站点附近
                            shiftInfo.setCurrentStationStatus("1"); //到站
                            shiftInfoService.update(shiftInfo);
                        }else {
                            //进入站点区域
                            //判断当前车辆是靠近还是远离站点
                            if (distance1>distance2) {
                                shiftInfo.setCurrentStationStatus("2"); //靠近
                                shiftInfoService.update(shiftInfo);
                            }
                            else if(distance1<distance2){
                                shiftInfo.setCurrentStationStatus("3"); //远离
                                shiftInfoService.update(shiftInfo);
                            }
                        }
                    }
                }catch (Exception ex){
                    log.error(ex.getMessage(),ex);
                }
            }
        }
    }

    @Override
    public void online(String deviceNo, String rawData) {
        VehicleInfo vehicleInfo = vehicleInfoService.findByGpsDeviceNo(deviceNo);

        if (vehicleInfo!=null){
            vehicleInfo.setOnline(true);
            vehicleInfo.setGpsUploadTime(new Date());
            vehicleInfo.setRawData(rawData);

            vehicleInfoService.update(vehicleInfo);
        }
    }
}
