package com.jpsoft.bus.modules.bus.service.impl;

import java.util.*;
import javax.annotation.Resource;

import com.jpsoft.bus.modules.base.entity.MergeOrderInfo;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.service.MergeOrderInfoService;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.bus.dto.StationStatusDTO;
import com.jpsoft.bus.modules.bus.entity.*;
import com.jpsoft.bus.modules.bus.service.*;
import com.jpsoft.bus.modules.merchant.entity.AccountInfo;
import com.sun.xml.bind.v2.TODO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import com.jpsoft.bus.modules.bus.dao.ShiftInfoDAO;
import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.github.pagehelper.PageHelper;

@Transactional
@Component(value="shiftInfoService")
public class ShiftInfoServiceImpl implements ShiftInfoService {
	@Resource(name="shiftInfoDAO")
	private ShiftInfoDAO shiftInfoDAO;

	@Autowired
	private GpsService gpsService;

	@Autowired
	private VehicleInfoService vehicleInfoService;

	@Autowired
	private CompanyInfoService companyInfoService;

	@Autowired
	private RouteAccountService routeAccountService;

	@Autowired
	private DriverInfoService driverInfoService;

	@Autowired
	private MergeOrderInfoService mergeOrderInfoService;

	@Autowired
	private OrderInfoService orderInfoService;

	@Autowired
	private PassengerInfoService passengerInfoService;

	@Autowired
	private StationInfoService stationInfoService;

	@Autowired
	private RabbitTemplate rabbitTemplate;

	@Autowired
	private PassengerMessageService passengerMessageService;

	@Override
	public ShiftInfo get(String id) {
		// TODO Auto-generated method stub
		return shiftInfoDAO.get(id);
	}

	@Override
	public int insert(ShiftInfo model) {
		// TODO Auto-generated method stub
		//model.setId(UUID.randomUUID().toString());

		return shiftInfoDAO.insert(model);
	}

	@Override
	public int update(ShiftInfo model) {
		// TODO Auto-generated method stub
		return shiftInfoDAO.update(model);
	}

	@Override
	public int delete(String id) {
		// TODO Auto-generated method stub
		return shiftInfoDAO.delete(id);
	}

	@Override
	public boolean exist(String id) {
		// TODO Auto-generated method stub
		int count = shiftInfoDAO.exist(id);

		return count > 0 ? true : false;
	}

	@Override
	public List<ShiftInfo> list() {
		// TODO Auto-generated method stub
		return shiftInfoDAO.list();
	}

	@Override
	public Page<ShiftInfo> pageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize,boolean count,List<Sort> sortList) {
        Page<ShiftInfo> page = PageHelper.startPage(pageNumber,pageSize,count).doSelectPage(()->{
            shiftInfoDAO.search(searchParams,sortList);
        });

        return page;
	}

	@Override
	public List<ShiftInfo> listSearch(Map<String, Object> searchParams, List<Sort> sortList) {
		 return shiftInfoDAO.search(searchParams,sortList);
	}

	@Override
	public List<ShiftInfo> findByDriverIdAndStatus(Long driverId, String status) {
		return shiftInfoDAO.findByDriverIdAndStatus(driverId,status);
	}

	@Override
	public List<ShiftInfo> findByVehicleIdAndStatus(String vehicleId, String status) {
		return shiftInfoDAO.findByVehicleIdAndStatus(vehicleId,status);
	}

	@Override
	public ShiftInfo createShift(VehicleInfo vehicleInfo, Long driverId, String startStationId, String endStationId) throws Exception {
		List<ShiftInfo> shiftInfoList = findByVehicleIdAndStatus(vehicleInfo.getId(),"1");
		if (shiftInfoList.size()>0){
			for (ShiftInfo shiftInfo : shiftInfoList){
				shiftInfo.setStatus("2");
				update(shiftInfo);
			}
		}
		//TODO 根据经纬度查询当前站点
		String currentStationId = gpsService.getLatelyStation(vehicleInfo.getId(),false);

		if (StringUtils.isEmpty(currentStationId)) {
			currentStationId = startStationId;
		}

		ShiftInfo shiftInfo = new ShiftInfo();
		shiftInfo.setId(UUID.randomUUID().toString());
		shiftInfo.setDriverId(Long.valueOf(driverId));
		shiftInfo.setVehicleId(vehicleInfo.getId());
		shiftInfo.setRouteId(vehicleInfo.getRouteId());
		shiftInfo.setStartStationId(startStationId);
		shiftInfo.setCurrentStationId(currentStationId);
		shiftInfo.setEndStationId(endStationId);
		shiftInfo.setStatus("1");
		shiftInfo.setCreateTime(new Date());
		insert(shiftInfo);

		return shiftInfo;
	}

	@Override
	public List<ShiftInfo> findByRouteIdAndStatus(String routeId, String status) {
		return shiftInfoDAO.findByRouteIdAndStatus(routeId,status);
	}

	@Override
	public List<ShiftInfo> findByRouteIdAndStatusAndStartStationId(String routeId, String status, String startStationId) {
		return shiftInfoDAO.findByRouteIdAndStatusAndStartStationId(routeId,status,startStationId);
	}

	@Override
	public void finishShift(ShiftInfo shiftInfo) {
		shiftInfo.setStatus("2");
		shiftInfo.setFinishTime(new Date());
		shiftInfo.setUpdateTime(new Date());
		update(shiftInfo);

		//修改当前班次下所有未下次乘客为下车状态
		List<PassengerInfo> passengerInfoList = passengerInfoService.findByShiftIdAndStatus(shiftInfo.getId(),"1");
		if (passengerInfoList.size()>0){
			for (PassengerInfo passengerInfo : passengerInfoList){
				passengerInfo.setStatus("2");
				passengerInfo.setDownTime(new Date());
				passengerInfo.setUpdateTime(new Date());
				passengerInfo.setUpdateBy(shiftInfo.getUpdateBy());
				passengerInfoService.update(passengerInfo);
			}
		}
	}

	@Override
	public void stationChange(ShiftInfo shiftInfo, String stationId) {
		StationInfo currentStation = stationInfoService.get(stationId);

		//站点发生变化时
		if (!shiftInfo.getCurrentStationId().equals(stationId)){
			shiftInfo.setCurrentStationId(stationId);
			shiftInfo.setUpdateTime(new Date());
			update(shiftInfo);

			rabbitTemplate.convertAndSend("stationRemindQueue", shiftInfo);

			//此线路的所有站点
			List<StationInfo> stationInfoList = stationInfoService.findByRouteId(shiftInfo.getRouteId());

			int currentStationIndex = currentStation.getSortNo();

			//线路终点站
			StationInfo endStation = stationInfoService.get(shiftInfo.getEndStationId());
			boolean frontToEnd = true;

			if(endStation.getId().equals(stationInfoList.get(0).getId())){
				//反向行驶
				frontToEnd = false;
			}

			//查询乘客表是否有过站的情况
			List<PassengerInfo> passengerInfoList = passengerInfoService.findByShiftStatusPayStatusNotTicketDown(shiftInfo.getId(),"1","1",stationId);

			if (passengerInfoList.size()>0){
				for (PassengerInfo passengerInfo : passengerInfoList){
					//乘客购票的下车站点
					StationInfo downStation = stationInfoService.get(passengerInfo.getTicketDownStationId());

					//按照线路顺行并且当前站点大于乘客购票站点，则过站
					if (frontToEnd){
						if(currentStationIndex > downStation.getSortNo()) {
							passengerInfo.setPayStatus("2");
						}
						else{
							passengerInfo.setPayStatus("1");
						}

						passengerInfoService.update(passengerInfo);
					}

					//按照线路逆行并且当前站点小于乘客购票站点，则过站
					if (!frontToEnd){
						if(currentStationIndex < downStation.getSortNo()) {
							passengerInfo.setPayStatus("2");
						}
						else {
							passengerInfo.setPayStatus("1");
						}

						passengerInfoService.update(passengerInfo);
					}

					//过站发消息
					if (passengerInfo.getPayStatus().equals("2")){
						List<OrderInfo> orderInfoList = orderInfoService.findByPassengerIdAndPayStatus(passengerInfo.getId(),20);
						OrderInfo orderInfo = orderInfoList.get(0);
						//综合账单
						MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(orderInfo.getMergeOrderId());
						if (StringUtils.isNotBlank(mergeOrderInfo.getOpenId())){
							PassengerMessage passengerMessage = passengerMessageService.findByPassengerId(orderInfo.getPassengerId());

							if (passengerMessage ==null){
								passengerMessage = new PassengerMessage();
								passengerMessage.setId(cn.hutool.core.lang.UUID.randomUUID().toString());
								passengerMessage.setOpenId(mergeOrderInfo.getOpenId());
								passengerMessage.setPassengerId(passengerInfo.getId());
								passengerMessage.setCreateTime(new Date());
								passengerMessageService.insert(passengerMessage);
							}
							else {
								passengerMessage.setReadStatus(false);
								passengerMessageService.update(passengerMessage);
							}
						}
					}
				}
			}

			//查询乘客有过站的情况
			List<PassengerInfo> passengerInfoList1 = passengerInfoService.findByShiftStatusPayStatusNotTicketDown(shiftInfo.getId(),"1","2",stationId);

			if (passengerInfoList1.size()>0){
				for (PassengerInfo passengerInfo : passengerInfoList1){
					List<OrderInfo> orderInfoList = orderInfoService.findByPassengerIdAndPayStatus(passengerInfo.getId(),20);
					OrderInfo orderInfo = orderInfoList.get(0);
					//综合账单
					MergeOrderInfo mergeOrderInfo = mergeOrderInfoService.get(orderInfo.getMergeOrderId());

					if (StringUtils.isNotBlank(mergeOrderInfo.getOpenId())){
						PassengerMessage passengerMessage = passengerMessageService.findByPassengerId(orderInfo.getPassengerId());
						if (passengerMessage ==null){
							passengerMessage = new PassengerMessage();
							passengerMessage.setId(cn.hutool.core.lang.UUID.randomUUID().toString());
							passengerMessage.setOpenId(mergeOrderInfo.getOpenId());
							passengerMessage.setPassengerId(passengerInfo.getId());
							passengerMessage.setCreateTime(new Date());
							passengerMessageService.insert(passengerMessage);
						}
						else {
							passengerMessage.setReadStatus(false);
							passengerMessageService.update(passengerMessage);
						}
					}
				}
			}
		}
	}

	@Override
	public List<String> findVehicleByDriverId(Long driverId) {
		return shiftInfoDAO.findVehicleByDriverId(driverId);
	}

	@Override
	public List<ShiftInfo> findByVehicleDateAccount(Date startDate, Date endDate, AccountInfo accountInfo) {

		List<ShiftInfo> list = new ArrayList<>();
		List<String> roleList = Arrays.asList(accountInfo.getRoles().split(","));

		for (String roleId : roleList) {
			if ("1".equals(roleId)){
				DriverInfo driverInfo = driverInfoService.findByPhone(accountInfo.getPhone());
				if (driverInfo != null){
					Map<String,Object> searchParams = new HashMap<>();
					searchParams.put("driverId",driverInfo.getId());
					if (startDate != null){
						searchParams.put("startTime",startDate);
					}
					if (endDate != null){
						searchParams.put("endTime",endDate);
					}
					List<Sort> sortList = new ArrayList<>();
					sortList.add(new Sort("a.create_time","desc"));
					List<ShiftInfo> shiftInfoList = listSearch(searchParams,sortList);
					if (shiftInfoList.size() >0) {
						for (ShiftInfo shiftInfo : shiftInfoList){
							if (!list.contains(shiftInfo)){
								list.add(shiftInfo);
							}
						}
					}
				}

			}
			else {
				List<VehicleInfo> vehicleInfoList = new ArrayList<>();
				if ("2".equals(roleId)){
					List<VehicleInfo> vehicleInfoList1 = vehicleInfoService.findByAccountId(accountInfo.getId());
					vehicleInfoList.addAll(vehicleInfoList1);
				}
				if ("3".equals(roleId)){
					List<RouteAccount> routeAccountList = routeAccountService.findByAccountId(accountInfo.getId());
					if (routeAccountList.size() > 0) {
						for (RouteAccount routeAccount : routeAccountList) {
							List<VehicleInfo> vehicleInfoList1 =vehicleInfoService.findByRouteId(routeAccount.getRouteId());
							vehicleInfoList.addAll(vehicleInfoList1);
						}
					}
				}
				if ("4".equals(roleId)){
					CompanyInfo companyInfo = companyInfoService.get(accountInfo.getCompanyId());
					List<VehicleInfo> vehicleInfoList1 = vehicleInfoService.findByCompanyId(companyInfo.getId());
					vehicleInfoList.addAll(vehicleInfoList1);
				}
				Map<String,Object> searchParams = new HashMap<>();
				searchParams.put("vehicleInfoList",vehicleInfoList);
				if (startDate != null){
					searchParams.put("startTime",startDate);
				}
				if (endDate != null){
					searchParams.put("endTime",endDate);
				}
				List<Sort> sortList = new ArrayList<>();
				sortList.add(new Sort("a.create_time","desc"));
				List<ShiftInfo> shiftInfoList = listSearch(searchParams,sortList);
				if (shiftInfoList.size() >0) {
					for (ShiftInfo shiftInfo : shiftInfoList){
						if (!list.contains(shiftInfo)){
							list.add(shiftInfo);
						}
					}
				}
			}

		}
		return list;
	}
}