package com.jpsoft.bus.modules.bus.service.impl;

import java.util.*;
import javax.annotation.Resource;

import com.jpsoft.bus.modules.bus.dto.VehicleDTO;
import com.jpsoft.bus.modules.bus.entity.*;
import com.jpsoft.bus.modules.bus.service.*;
import com.jpsoft.bus.modules.merchant.entity.AccountInfo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import com.jpsoft.bus.modules.bus.dao.VehicleInfoDAO;
import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.github.pagehelper.PageHelper;

@Transactional
@Component(value = "vehicleInfoService")
public class VehicleInfoServiceImpl implements VehicleInfoService {
    @Resource(name = "vehicleInfoDAO")
    private VehicleInfoDAO vehicleInfoDAO;

    @Autowired
    private DriverInfoService driverInfoService;

    @Autowired
    private ShiftInfoService shiftInfoService;

    @Autowired
    private CompanyInfoService companyInfoService;

    @Autowired
    private RouteAccountService routeAccountService;

    @Override
    public VehicleInfo get(String id) {
        // TODO Auto-generated method stub
        return vehicleInfoDAO.get(id);
    }

    @Override
    public int insert(VehicleInfo model) {
        // TODO Auto-generated method stub
        //model.setId(UUID.randomUUID().toString());

        return vehicleInfoDAO.insert(model);
    }

    @Override
    public int update(VehicleInfo model) {
        // TODO Auto-generated method stub
        return vehicleInfoDAO.update(model);
    }

    @Override
    public int delete(String id) {
        // TODO Auto-generated method stub
        return vehicleInfoDAO.delete(id);
    }

    @Override
    public boolean exist(String id) {
        // TODO Auto-generated method stub
        int count = vehicleInfoDAO.exist(id);

        return count > 0 ? true : false;
    }

    @Override
    public List<VehicleInfo> list() {
        // TODO Auto-generated method stub
        return vehicleInfoDAO.list();
    }

    @Override
    public Page<VehicleInfo> pageSearch(Map<String, Object> searchParams, int pageNumber, int pageSize, boolean count, List<Sort> sortList) {
        Page<VehicleInfo> page = PageHelper.startPage(pageNumber, pageSize, count).doSelectPage(() -> {
            vehicleInfoDAO.search(searchParams, sortList);
        });

        return page;
    }

    @Override
    public VehicleInfo findByCodeAndCarNum(String activationCode, String carNum) {
        return vehicleInfoDAO.findByCodeAndCarNum(activationCode, carNum);
    }

    @Override
    public VehicleInfo findByGpsDeviceNo(String gpsDeviceNo) {
        return vehicleInfoDAO.findByGpsDeviceNo(gpsDeviceNo);
    }

    @Override
    public void updateGps(VehicleInfo v) {
        vehicleInfoDAO.updateGps(v);
    }

    @Override
    public List<VehicleInfo> findByCompanyId(String companyId) {
        return vehicleInfoDAO.findByCompanyId(companyId);
    }

    @Override
    public VehicleInfo findByCarNum(String carNum) {
        return vehicleInfoDAO.findByCarNum(carNum);
    }

    @Override
    public List<VehicleInfo> getRunningList() {
        return vehicleInfoDAO.getRunningList();
    }

    @Override
    public List<VehicleInfo> findByRouteId(String routeId) {
        return vehicleInfoDAO.findByRouteId(routeId);
    }

    @Override
    public List<VehicleInfo> findByAccountId(String accountId) {
        return vehicleInfoDAO.findByAccountId(accountId);
    }

    @Override
    public List<VehicleInfo> getAllVehicleByAccount(AccountInfo accountInfo) {

        List<VehicleInfo> list = new ArrayList<>();


        if (StringUtils.isNotBlank(accountInfo.getRoles())) {
            //账户角色
            List<String> roleList = Arrays.asList(accountInfo.getRoles().split(","));

            for (String roleId : roleList) {
                //司机
                if ("1".equals(roleId)) {
                    DriverInfo driverInfo = driverInfoService.findByPhone(accountInfo.getPhone());
                    if (driverInfo != null) {
                        List<String> vehicleIdList = shiftInfoService.findVehicleByDriverId(driverInfo.getId());
                        if (vehicleIdList.size() > 0) {
                            for (String vehicleId : vehicleIdList) {
                                VehicleInfo vehicleInfo = get(vehicleId);
                                if (!list.contains(vehicleInfo)) {
                                    list.add(vehicleInfo);
                                }
                            }
                        }
                    }
                }
                //车主
                if ("2".equals(roleId)) {
                    List<VehicleInfo> vehicleInfoList = findByAccountId(accountInfo.getId());
                    if (vehicleInfoList.size() > 0) {
                        for (VehicleInfo vehicleInfo : vehicleInfoList) {
                            if (!list.contains(vehicleInfo)) {
                                list.add(vehicleInfo);
                            }
                        }
                    }
                }
                //车队长
                if ("3".equals(roleId)) {
                    List<RouteAccount> routeAccountList = routeAccountService.findByAccountId(accountInfo.getId());
                    if (routeAccountList.size() > 0) {
                        for (RouteAccount routeAccount : routeAccountList) {
                            List<VehicleInfo> vehicleInfoList1 = findByRouteId(routeAccount.getRouteId());
                            if (vehicleInfoList1.size() > 0) {
                                for (VehicleInfo vehicleInfo : vehicleInfoList1) {
                                    if (!list.contains(vehicleInfo)) {
                                        list.add(vehicleInfo);
                                    }
                                }
                            }
                        }
                    }

                }
                //管理人员
                if ("4".equals(roleId)) {
                    CompanyInfo companyInfo = companyInfoService.get(accountInfo.getCompanyId());
                    List<VehicleInfo> vehicleInfoList = findByCompanyId(companyInfo.getId());

                    if (vehicleInfoList.size() > 0) {
                        for (VehicleInfo vehicleInfo : vehicleInfoList) {
                            if (!list.contains(vehicleInfo)) {
                                list.add(vehicleInfo);
                            }
                        }

                    }
                }

            }
        }
			/*CompanyInfo companyInfo = companyInfoService.get(accountInfo.getCompanyId());
			List<VehicleInfo> vehicleInfoList = findByCompanyId(companyInfo.getId());

			if (vehicleInfoList.size()>0){
				for (VehicleInfo vehicleInfo : vehicleInfoList){
					list.add(vehicleInfo);
				}
			}

			List<RouteAccount> routeAccountList = routeAccountService.findByAccountId(accountInfo.getId());
			if (routeAccountList.size()>0){
				for (RouteAccount routeAccount : routeAccountList){
					List<VehicleInfo> vehicleInfoList1 = findByRouteId(routeAccount.getRouteId());
					if (vehicleInfoList1.size()>0){
						for (VehicleInfo vehicleInfo:vehicleInfoList1){
							if (!list.contains(vehicleInfo)){
								list.add(vehicleInfo);
							}
						}
					}
				}
			}

			List<VehicleInfo> vehicleInfoList2 = findByAccountId(accountInfo.getId());
			if (vehicleInfoList2.size() > 0){
				for (VehicleInfo vehicleInfo : vehicleInfoList2){
					if (!list.contains(vehicleInfo)){
						list.add(vehicleInfo);
					}
				}

			}
		}*/


        return list;
    }
}