package com.jpsoft.gps.handler;

import com.jpsoft.gps.callback.GpsDataCallback;
import com.jpsoft.gps.utils.GPSUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;
import java.text.DecimalFormat;

@Slf4j
public class ProcessHandler extends SimpleChannelInboundHandler<String> {
    private GpsDataCallback callback;

    public ProcessHandler(GpsDataCallback callback){
        this.callback = callback;
    }

    @Override
    public void channelRead0(ChannelHandlerContext ctx, String msg) throws Exception {
        //#20201021+30.309052;112.270986+\n

        String[] arr = msg.split("\\+");

        if (arr.length>=2) {
            String deviceNo = arr[0];
            String[] subArr = arr[1].split(";");

            //如果小数部分超过6位，则需要处理
            String lat = fixOverflow(subArr[0]);
            String lng = fixOverflow(subArr[1]);

            if (subArr.length==3){
                //测试用不进行转换
                if (this.callback != null) {
                    this.callback.receive(deviceNo, lng, lat,msg,false);
                }
            }
            else {
                double wd = Double.valueOf(lat);
                double jd = Double.valueOf(lng);

                DecimalFormat df = new DecimalFormat("#.######");

                if (wd > 0 && jd > 0) {
                    //gps坐标转高德地图坐标系 gcj02
                    double[] gcj = GPSUtil.gps84_To_Gcj02(wd, jd);//坐标转换

                    System.out.println(deviceNo + "," + df.format(gcj[1]) + "," + df.format(gcj[0]));

                    if (this.callback != null) {
                        new Thread(()->{
                            this.callback.receive(deviceNo, df.format(gcj[1]), df.format(gcj[0]),msg,false);
                        }).start();
                    }
                }
                else{
                    if (this.callback != null) {
                        new Thread(()->{
                            this.callback.online(deviceNo,msg);
                        }).start();
                    }
                }
            }
        }
    }

    private String fixOverflow(String value) {
        String[] arr = value.split("\\.");

        if (arr.length>1) {
            if (arr[1].length() > 6) {
                arr[1] = arr[1].substring(1);
            }

            return arr[0] + "." + arr[1];
        }
        else{
            return value;
        }
    }
}
