package com.jpsoft.bus.modules.bus.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.Page;
import com.jpsoft.bus.config.OSSConfig;
import com.jpsoft.bus.modules.base.entity.OrderInfo;
import com.jpsoft.bus.modules.base.service.OrderInfoService;
import com.jpsoft.bus.modules.common.dto.MessageResult;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.bus.modules.bus.entity.PassengerInfo;
import com.jpsoft.bus.modules.bus.service.PassengerInfoService;
import com.jpsoft.bus.modules.common.utils.OSSUtil;
import com.jpsoft.bus.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 乘客
 */

@RestController
@RequestMapping("/bus/passengerInfo")
@Api(description = "passengerInfo")
public class PassengerInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private PassengerInfoService passengerInfoService;

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private OSSConfig ossConfig;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<PassengerInfo> create(){
        MessageResult<PassengerInfo> msgResult = new MessageResult<>();

        PassengerInfo passengerInfo = new PassengerInfo();

        msgResult.setData(passengerInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<PassengerInfo> add(@RequestBody PassengerInfo passengerInfo,@RequestAttribute String subject){
        MessageResult<PassengerInfo> msgResult = new MessageResult<>();

        try {
            passengerInfo.setId(UUID.randomUUID().toString());
            passengerInfo.setDelFlag(false);
            passengerInfo.setCreateBy(subject);
            passengerInfo.setCreateTime(new Date());
            
            int affectCount = passengerInfoService.insert(passengerInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(passengerInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<PassengerInfo> edit(@PathVariable("id") String id){
        MessageResult<PassengerInfo> msgResult = new MessageResult<>();

        try {
            PassengerInfo passengerInfo = passengerInfoService.get(id);

            if (passengerInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(passengerInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<PassengerInfo> update(@RequestBody PassengerInfo passengerInfo,@RequestAttribute String subject){
        MessageResult<PassengerInfo> msgResult = new MessageResult<>();

        try {
            passengerInfo.setUpdateBy(subject);
            passengerInfo.setUpdateTime(new Date());
            
            int affectCount = passengerInfoService.update(passengerInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(passengerInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PassengerInfo passengerInfo = passengerInfoService.get(id);
            passengerInfo.setDelFlag(true);
            passengerInfo.setUpdateBy(subject);
            passengerInfo.setUpdateTime(new Date());

            int affectCount = passengerInfoService.update(passengerInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                PassengerInfo passengerInfo = passengerInfoService.get(id);
                passengerInfo.setDelFlag(true);
                passengerInfo.setUpdateBy(subject);
                passengerInfo.setUpdateTime(new Date());

                affectCount += passengerInfoService.update(passengerInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            @RequestParam(value="vehicleShiftId",defaultValue="") String vehicleShiftId,
            @RequestParam(value="vehiclePersonId",defaultValue="") String vehiclePersonId,
            @RequestParam(value="userName",defaultValue="") String userName,
            @RequestParam(value="payStatus",defaultValue="") String payStatus,
            @RequestParam(value="status",defaultValue="") String status,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(vehicleShiftId)) {
            searchParams.put("vehicleShiftId",vehicleShiftId);
        }

        if (StringUtils.isNotEmpty(vehiclePersonId)) {
            searchParams.put("vehiclePersonId",Long.valueOf(vehiclePersonId));
        }

        if (StringUtils.isNotEmpty(userName)) {
            searchParams.put("userName","%" + userName + "%");
        }

        if (StringUtils.isNotEmpty(payStatus)) {
            searchParams.put("payStatus",payStatus);
        }

        if (StringUtils.isNotEmpty(status)) {
            searchParams.put("status",status);
        }

        Page<PassengerInfo> page = passengerInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        for (PassengerInfo passengerInfo:page) {
            OrderInfo orderInfo = orderInfoService.findByPassengerId(passengerInfo.getId());

            if(orderInfo!=null){
                if(("wechat").equals(orderInfo.getPayName())){
                    passengerInfo.setPayName("微信");
                }
                else if(("alipay").equals(orderInfo.getPayName())){
                    passengerInfo.setPayName("支付宝");
                }
                else if(("cash").equals(orderInfo.getPayName())){
                    passengerInfo.setPayName("现金");
                }
                else if(("ticket").equals(orderInfo.getPayName())){
                    passengerInfo.setPayName("车票");
                }
            }

            List<OrderInfo> orderInfoList = orderInfoService.findByPassengerIdAndPayStatus(passengerInfo.getId(), 20);

            BigDecimal totalFee = new BigDecimal(0);

            //乘客所有购票
            for (OrderInfo orderInfo1 : orderInfoList) {
                totalFee = totalFee.add(orderInfo1.getTotalFee());
            }

            passengerInfo.setTotalFee(totalFee);
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="导出后勤月报表")
    @PostMapping("exportXls")
    public MessageResult<String> exportXls(
            @RequestBody JSONObject requestBody,
            @RequestAttribute String subject){
        MessageResult<String> msgResult = new MessageResult<>();

        try {
            XSSFWorkbook workbook = new XSSFWorkbook();
            XSSFSheet sheet = workbook.createSheet();

            //表头
            XSSFRow rowTitle = sheet.createRow(0);

            rowTitle.createCell(0).setCellValue("序号");

            rowTitle.createCell(1).setCellValue("乘车状态");

            rowTitle.createCell(2).setCellValue("购票状态");

            rowTitle.createCell(3).setCellValue("购票方式");

            rowTitle.createCell(4).setCellValue("金额");

            rowTitle.createCell(5).setCellValue("人员编号");

            rowTitle.createCell(6).setCellValue("购票起始站");

            rowTitle.createCell(7).setCellValue("购票目的站");

            rowTitle.createCell(8).setCellValue("下车站点");

            rowTitle.createCell(9).setCellValue("上车时间");

            rowTitle.createCell(10).setCellValue("下车时间");

            rowTitle.createCell(11).setCellValue("记录时间");

            JSONArray array = requestBody.getJSONArray("data");

            for(int i=0; i<array.size(); i++) {
                Row row = sheet.createRow(i+1);

                JSONObject jsonObject = array.getJSONObject(i);

                int colIndex = 0;

                row.createCell(colIndex).setCellValue(i+1);
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getString("statusName"));
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getString("payStatusName"));
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getString("payName"));
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getDoubleValue("totalFee"));
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getString("vehiclePersonId"));
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getString("ticketUpStationName"));
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getString("ticketDownStationName"));
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getString("downStationName"));
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getString("upTime"));
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getString("downTime"));
                colIndex++;

                row.createCell(colIndex).setCellValue(jsonObject.getString("createTime"));
                colIndex++;

                //todo 将wb保存到oss
                ByteArrayOutputStream output = new ByteArrayOutputStream();
                workbook.write(output);

                byte[] buffer = output.toByteArray();
                ByteArrayInputStream input = new ByteArrayInputStream(buffer);

                //格式化
                SimpleDateFormat sim = new SimpleDateFormat("yyyyMMddHHmmss");
                String fileName = "乘客列表" + sim.format(new Date()) + ".xlsx";
                String downloadUrl = OSSUtil.upload(ossConfig, "passenger", fileName, input);

                msgResult.setData(downloadUrl);
                msgResult.setResult(true);
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
