package com.jpsoft.bus;

import cn.hutool.core.lang.UUID;
import com.github.pagehelper.Page;
import com.jpsoft.bus.modules.bus.entity.GpsDataInfo;
import com.jpsoft.bus.modules.bus.service.GpsDataInfoService;
import com.jpsoft.bus.modules.common.dto.Sort;
import com.jpsoft.gps.utils.GPSUtil;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

@SpringBootTest
class SmartBusServerApplicationTests {
    @Autowired
    private GpsDataInfoService gpsDataInfoService;

    @Test
    void contextLoads() {
    }

    @Test
    void repairGpsDataFromLog(){
        //2021-05-26 15:47:06,370 WARN [nioEventLoopGroup-5-2] com.jpsoft.gps.codec.CustomDecoder [CustomDecoder.java : 27] 解析数据:971B+29.0016077;111.568507+

        try {
            FileInputStream input = new FileInputStream("D:\\logfile.log");
            InputStreamReader bis = new InputStreamReader(input);
            BufferedReader reader = new BufferedReader(bis);

            String line = null;
            String prefix = "解析数据:971B";
            DecimalFormat df = new DecimalFormat("#.######");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

            int total = 0;
            int error = 0;

            while ((line = reader.readLine()) != null){
                int index = line.indexOf(prefix);

                if (index!=-1){
                    String time = line.substring(0,19);
                    String location = line.substring(index + prefix.length());

                    String[] arr = location.substring(1,location.length()-1).split(";");

                    if (Double.valueOf(arr[0])==0){
                        continue;
                    }

                    String[] latitude = arr[0].split("\\.");

                    if (latitude[1].length()>6){
                        System.out.println("纬度小数位数超过了6位," + arr[0]);
                        latitude[1] = latitude[1].substring(1);

                        error++;
                    }

                    double[] gcj = GPSUtil.gps84_To_Gcj02(
                            Double.valueOf(latitude[0] + "." + latitude[1]),
                            Double.valueOf(arr[1]));//坐标转换

                    System.out.println(time + "\t" + df.format(gcj[1]) + "," + df.format(gcj[0]));

                    total++;

                    //写入数据库
                    GpsDataInfo gpsDataInfo = new GpsDataInfo();
                    gpsDataInfo.setId(UUID.randomUUID().toString());
                    gpsDataInfo.setDeviceNo("014144732509");
                    gpsDataInfo.setCreateTime(sdf.parse(time));
                    gpsDataInfo.setLatitude(df.format(gcj[0]));
                    gpsDataInfo.setLongitude(df.format(gcj[1]));
                    gpsDataInfo.setDelFlag(false);

                    gpsDataInfoService.insert(gpsDataInfo);
                }
            }

            System.out.println("总数：" + total + ",错误=" + error);
        }
        catch (Exception ex){
            ex.printStackTrace();
        }
    }

    @Test
    void repairGpsData(){
        Map<String,Object> searchParams = new HashMap<>();

        String deviceNo = "971B";

        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date startTime = sdf.parse("2021-05-26 15:46:26");
            Date endTime = sdf.parse("2021-05-26 16:00:00");

            searchParams.put("deviceNo", deviceNo);

            if (startTime != null) {
                searchParams.put("startTime", startTime);
            }

            if (endTime != null) {
                searchParams.put("endTime", endTime);
            }

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time", "asc"));

            //查询某段时间时间
            Page<GpsDataInfo> page = gpsDataInfoService.pageSearch(searchParams, 1, 1000, true, sortList);

            for (GpsDataInfo gpsDataInfo : page.getResult()) {
                double[] gps84 = GPSUtil.gcj02_To_Gps84(Double.valueOf(gpsDataInfo.getLatitude()),
                        Double.valueOf(gpsDataInfo.getLongitude()));//坐标转换

                System.out.println(sdf.format(gpsDataInfo.getCreateTime()) + "->" + gps84[1] + "," + gps84[0]);
            }
        }
        catch (Exception ex){
            ex.printStackTrace();
        }
    }
}
